/* -*- Mode: IDL; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is written by Juan Lang.
 *
 * The Initial Developer of the Original Code is 
 * Juan Lang.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or 
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
#ifndef nsNetInterfaceDetector_h__
#define nsNetInterfaceDetector_h__

#include "nsINetInterfaceDetector.h"
#include "nsIRunnable.h"
#include "nsIObserver.h"
#include "nsIThread.h"
#include "nsCOMPtr.h"

class nsNetInterfaceDetector : public nsINetInterfaceDetector,
                               public nsIRunnable,
                               public nsIObserver
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSINETINTERFACEDETECTOR
  NS_DECL_NSIRUNNABLE
  NS_DECL_NSIOBSERVER

  nsNetInterfaceDetector();
  virtual ~nsNetInterfaceDetector();

protected:
  struct MyEvent : public PLEvent {
      MyEvent(const char *aEventID) : mEventID(aEventID) { }
      const char *mEventID;
  };

  static nsresult SendEventToUI(const char *aEventID);
  static void * PR_CALLBACK HandleInterfaceEvent(PLEvent *aEvent);
  static void   PR_CALLBACK DestroyInterfaceEvent(PLEvent *aEvent);

  static nsNetInterfaceDetector *mInstance;
  nsCOMPtr<nsIThread>            mThread;
  PRLock *                       mLock;
  PRCondVar *                    mCondVar;

#ifdef XP_WIN
  static LRESULT NS_STDCALL WndProc(HWND, UINT, WPARAM, LPARAM);
  static VOID NS_STDCALL TimerCallback(HWND, UINT, UINT_PTR, DWORD);
  static DWORD GetOperationalStatus(DWORD aAdapterIndex);
  static PRBool InternalHaveNetworkInterfaceNT(PRBool *aWaitingOnDHCP);
  static PRBool InternalHaveNetworkInterfaceDefault(PRBool *aWaitingOnDHCP);
  static PRBool InternalHaveNetworkInterface(PRBool *aWaitingOnDHCP);

  HWND    mWindow;
  DWORD   mStopPollingTime;
  PRUintn mNumChecking;
  enum { ValidInterface, InitialDelay, DHCPDelay, NoValidInterface } mState;
#endif
};

#endif /* nsNetInterfaceDetector_h__ */
